@@ -1,3 +1,7 @@
+# Copyright (C) 2009 Sebastian Rahlf <basti at redtoad dot de>
+#
+# This program is release under the BSD License. You can find the full text of
+# the license in the LICENSE file.
 
 """
 Amazon Product Advertising API
@@ -27,14 +31,17 @@ secret key.
 You'll also need the python module lxml (>=2.1.5) and, if you're using python 
 2.4, also pycrypto.
 
-Kudos
------
+License
+-------
 
-The ``_build_url()`` function is based on code by Adam Cox (found at 
-http://blog.umlungu.co.uk/blog/2009/jul/12/pyaws-adding-request-authentication/)
+This program is release under the BSD License. You can find the full text of
+the license in the LICENSE file.
 
 """
 
+__version__ = '0.2.3'
+__docformat__ = "restructuredtext en"
+
 from base64 import b64encode
 from datetime import datetime, timedelta
 
@@ -49,10 +56,7 @@ import re
 import socket
 from time import strftime, gmtime
 from urlparse import urlsplit
-from urllib import quote
-from urllib2 import urlopen, HTTPError
-
-__docformat__ = "restructuredtext en"
+from urllib2 import quote, urlopen, HTTPError
 
 LOCALES = {
     'ca' : 'http://ecs.amazonaws.ca/onca/xml', 
@@ -140,18 +144,27 @@ class NotEnoughParameters (Exception):
     Your request should have at least one parameter which you did not submit.
     """
 
+class InvalidParameterCombination (Exception):
+    """
+    Your request contained a restricted parameter combination.
+    """
+
 INVALID_VALUE_REG = re.compile(
     'The value you specified for (?P<parameter>\w+) is invalid.')
 
 INVALID_PARAMETER_VALUE_REG = re.compile('(?P<value>.+?) is not a valid value '
     'for (?P<parameter>\w+). Please change this value and retry your request.')
 
-NOSIMILARITIES_REG = re.compile('There are no similar items for this ASIN: '
-                                '(?P<ASIN>\w+).')
+NOSIMILARITIES_REG = re.compile(
+    'There are no similar items for this ASIN: (?P<ASIN>\w+).')
 
 NOT_ENOUGH_PARAMETERS_REG = re.compile('Your request should have atleast '
         '(?P<numer>\d+) of the following parameters: (?P<parameters>[\w ,]+).')
 
+INVALID_PARAMETER_COMBINATION_REG = re.compile(
+     'Your request contained a restricted parameter combination.'
+     '\s*(?P<message>\w.*)$') # only the last bit is of interest here
+
 class API (object):
     
     """
@@ -170,15 +183,40 @@ class API (object):
         rating = root.Items.Item.CustomerReviews.AverageRating.pyval
         total_reviews = root.Items.Item.CustomerReviews.TotalReviews.pyval
         review_pages = root.Items.Item.CustomerReviews.TotalReviewPages.pyval
-
+        
+    It is possible to use a different module for parsing the XML response. For 
+    instance, you can use ``xml.minidom`` instead of ``lxml`` by defining a
+    custom result processor::
+        
+        def minidom_response_parser(fp):
+            root = parse(fp)
+            # parse errors
+            for error in root.getElementsByTagName('Error'):
+                code = error.getElementsByTagName('Code')[0].firstChild.nodeValue
+                msg = error.getElementsByTagName('Message')[0].firstChild.nodeValue
+                raise AWSError(code, msg)
+            return root
+        api = API(AWS_KEY, SECRET_KEY, processor=minidom_response_parser)
+        root = api.item_lookup('0718155157')
+        print root.toprettyxml()
+        # ...
+        
+    Just make sure it raises an ``AWSError`` with the appropriate error code 
+    and message.
     """
     
-    VERSION = '2009-10-01' #: supported Amazon API version
+    VERSION = '2009-11-01' #: supported Amazon API version
     REQUESTS_PER_SECOND = 1 #: max requests per second
     TIMEOUT = 5 #: timeout in seconds
     
-    def __init__(self, access_key_id, secret_access_key, locale='de'):
-        
+    def __init__(self, access_key_id, secret_access_key, 
+                 locale='de', processor=None):
+        """
+        :param access_key_id: AWS access key ID.
+        :param secret_key_id: AWS secret key.
+        :param locale: localise results by using one value from ``LOCALES``.
+        :param processor: result processing function (``None`` if unsure).
+        """
         self.access_key = access_key_id
         self.secret_key = secret_access_key
         
@@ -193,9 +231,13 @@ class API (object):
         self.last_call = datetime(1970, 1, 1)
         self.throttle = timedelta(seconds=1)/self.REQUESTS_PER_SECOND
         
+        self.response_processor = processor
+        
     def _build_url(self, **qargs):
         """
-        Builds a signed URL for querying Amazon AWS.
+        Builds a signed URL for querying Amazon AWS.  This function is based
+        on code by Adam Cox (found at
+        http://blog.umlungu.co.uk/blog/2009/jul/12/pyaws-adding-request-authentication/)
         """
         # remove empty (=None) parameters
         for key, val in qargs.items():
@@ -252,8 +294,13 @@ class API (object):
     
     def _parse(self, fp):
         """
-        Calls the Amazon Product Advertising API and objectifies the response.
+        Processes the AWS response (file like object). XML is fed in, some 
+        usable output comes out. It will use a different result_processor if 
+        you have defined one. 
         """
+        if self.response_processor:
+            return self.response_processor(fp)
+        
         tree = objectify.parse(fp)
         root = tree.getroot()
         
@@ -265,10 +312,15 @@ class API (object):
                          namespaces={'aws' : nspace})
         
         for error in errors:
+            
             if error.Code.text == 'AWS.InvalidParameterValue':
                 m = INVALID_PARAMETER_VALUE_REG.search(error.Message.text)
                 raise InvalidParameterValue(m.group('parameter'), m.group('value'))
             
+            if error.Code.text == 'AWS.RestrictedParameterValueCombination':
+                m = INVALID_PARAMETER_COMBINATION_REG.search(error.Message.text)
+                raise InvalidParameterCombination(m.group('message'))
+
             raise AWSError(error.Code.text, error.Message.text)
         
         #~ from lxml import etree
@@ -347,7 +399,7 @@ class API (object):
         except AWSError, e:
             
             if (e.code=='AWS.InvalidEnumeratedParameter' 
-            and INVALID_SEARCH_INDEX_REG.search(e.msg)):
+            and INVALID_VALUE_REG.search(e.msg)):
                 raise InvalidSearchIndex(search_index)
             
             if e.code=='AWS.InvalidResponseGroup': 
@@ -631,16 +683,16 @@ class ResultPaginator (object):
             
             if self.nspace is None:
                 self.nspace = root.nsmap.get(None, '')
-        
-            current_page = self.get_current_page_numer(root)
-            total_results = self.get_total_results(root)
-            total_pages = self.get_total_page_numer(root)
+            
+            current_page = self._get_current_page_numer(root)
+            total_results = self._get_total_results(root)
+            total_pages = self._get_total_page_numer(root)
             
             yield root
             
             kwargs[self.counter] += 1
             
-    def get_total_page_numer(self, root):
+    def _get_total_page_numer(self, root):
         """
         Get total number of paginator pages.
         """
@@ -650,7 +702,7 @@ class ResultPaginator (object):
         except IndexError:
             return None
         
-    def get_current_page_numer(self, root):
+    def _get_current_page_numer(self, root):
         """
         Get number of current paginator page.
         """
@@ -660,7 +712,7 @@ class ResultPaginator (object):
         except IndexError:
             return None
     
-    def get_total_results(self, root):
+    def _get_total_results(self, root):
         """
         Get total number of results.
         """
